@@ -7,14 +7,7 @@
  * modification of this software and in all copies of the supporting
  * documentation for such software.
  *
- * This software is subject to United States export controls.  You may
- * not export it, in whole or in part, or cause or allow such export,
- * through act or omission, without prior authorization from the United
- * States government and written permission from AT&T.  In particular,
- * you may not make any part of this software available for general or
- * unrestricted distribution to others, nor may you disclose this software
- * to persons other than citizens and permanent residents of the United
- * States and Canada. 
+ * This software is subject to United States export controls.
  *
  * THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  * WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
@@ -55,6 +48,7 @@
 #define SKIPJACK 4	/* 2 key hybrid SKIPJACK (PCMCIA) (n/a) */
 #define MCG 5		/* 1 key hybrid MacGuffin */
 #define SAFER_SK128 6	/* 1 key hybrid SAFER-SK128 */
+#define TRUE_THREE_DES 7 /* 3 key hybrid 3DES */
 
 typedef struct fhdata {
 	u_char magic[8];/* 0x0123456789abcdef */
@@ -115,12 +109,20 @@ typedef struct cfskey {  /* now holds expanded keys for DES also */
 			u_char primary[128];
 			u_char secondary[128];
 		} des;
-		struct { /* 3DES */
+		struct { /* two-key 3DES */
 			u_char primary1[128];
 			u_char primary2[128];
 			u_char secondary1[128];
 			u_char secondary2[128];
 		} des3;
+		struct { /* three-key 3DES */
+			u_char primary1[128];
+			u_char primary2[128];
+			u_char primary3[128];
+			u_char secondary1[128];
+			u_char secondary2[128];
+			u_char secondary3[128];
+		} dest3;
 		struct { /* MacGuffin */
 			mcg_key primary;
 			mcg_key secondary;
