@@ -7,14 +7,7 @@
  * modification of this software and in all copies of the supporting
  * documentation for such software.
  *
- * This software is subject to United States export controls.  You may
- * not export it, in whole or in part, or cause or allow such export,
- * through act or omission, without prior authorization from the United
- * States government and written permission from AT&T.  In particular,
- * you may not make any part of this software available for general or
- * unrestricted distribution to others, nor may you disclose this software
- * to persons other than citizens and permanent residents of the United
- * States and Canada. 
+ * This software is subject to United States export controls.
  *
  * THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  * WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
@@ -23,7 +16,7 @@
  */
 
 /*
- * client cfs mkdir - 1.3.1
+ * client cfs mkdir - 1.3.2
  */
 #include <stdio.h>
 #include <rpc/rpc.h>
@@ -40,6 +33,7 @@ main(argc,argv)
 	char *pw;
 	char pword[256];
 	char *getpassword();
+	int n;
 	cfs_admkey k;
 	cfskey kt;
 	char path[1024];
@@ -49,19 +43,27 @@ main(argc,argv)
 	struct timeval tv;
 	u_long r;
 	int i;
-	int ciph=CFS_STD_DES;
+	int ciph=CFS_THREE_DES;
 	int cfmt=1;
 	int smsize=LARGESMSIZE;
 	unsigned char ekey[128];
 	unsigned char ek1[128];
 	unsigned int l;
+	int keycheck=1;
 	
 	while (--argc && (**++argv == '-')) {
 		for (flg= ++*argv; *flg; ++flg)
 			switch (*flg) {
-			    case '3':
+			    case '1':
+			    case 'd':
+				ciph=CFS_STD_DES;
+				break;
+			    case '2':
 				ciph=CFS_THREE_DES;
 				break;
+			    case '3':
+				ciph=CFS_TRUE_THREE_DES;
+				break;
 #ifdef NOTDEF
 			    case 'b':
 				ciph=CFS_BLOWFISH;
@@ -69,9 +71,13 @@ main(argc,argv)
 #endif				
 			    case 'm':
 				ciph=CFS_MACGUFFIN;
+				fprintf(stderr,
+				      "Warning: MacGuffin is a weak cipher\n");
 				break;
 			    case 's':
 				ciph=CFS_SAFER_SK128;
+				fprintf(stderr,
+				      "Warning: SAFER is a new cipher.\n");
 				break;
 			    case 'o':
 				cfmt=0;
@@ -79,32 +85,48 @@ main(argc,argv)
 			    case 'p':	/* puny memory option */
 				smsize=SMALLSMSIZE;
 				break;
+			    case '-':
+				keycheck=0;
+				break;
 			    default:
-				fprintf(stderr,"usage: cmkdir [-3bmosp] dir\n");
+				fprintf(stderr,"usage: cmkdir [-123bdmosp-] dir\n");
 				exit(1);
 			}
 	}
 	if (argc!=1) {
-		fprintf(stderr,"Usage: cmkdir [-3bmost] dir\n");
+		fprintf(stderr,"Usage: cmkdir [-123bdmosp-] dir\n");
 		exit(1);
 	}
-	if ((pw=getpassword("Key:"))==NULL) {
-		fprintf(stderr,"Can't get key\n");
-		exit(1);
-	}
-	strcpy(pword,pw);
-	if (strlen(pw)<16) {
-		fprintf(stderr,"Key must be at least 16 chars.\n");
-		exit(1);
-	}
-	if ((pw=getpassword("Again:"))==NULL) {
-		fprintf(stderr,"Can't get key\n");
-		exit(1);
-	}
-	if (strcmp(pword,pw)!=0) {
-		fprintf(stderr,
+	if (keycheck) {
+		if ((pw=getpassword("Key:"))==NULL) {
+			fprintf(stderr,"Can't get key\n");
+			exit(1);
+		}
+		strcpy(pword,pw);
+		if (strlen(pw)<16) {
+			fprintf(stderr,"Key must be at least 16 chars.\n");
+			exit(1);
+		}
+		if ((pw=getpassword("Again:"))==NULL) {
+			fprintf(stderr,"Can't get key\n");
+			exit(1);
+		}
+		if (strcmp(pword,pw)!=0) {
+		     fprintf(stderr,
 			"Keys don't match; drink some coffee and try again\n");
-		exit(1);
+		     exit(1);
+		}
+	}
+	else { /* just accept key from stdio */
+		if (fgets(pword,256,stdin) == NULL) {
+			perror("cmkdir");
+			exit(1);
+		}
+		pw=pword;
+		pw[255]='\0';
+		n=strlen(pw);
+		if ((n>0) && (pw[n-1] == '\n'))
+			pw[n-1] = '\0';
 	}
 	if (smsize != LARGESMSIZE)
 		sprintf(pw,"%s%d",pword,smsize);
@@ -137,8 +159,7 @@ main(argc,argv)
 	sprintf(path,"%s/...",argv[0]);
 	strcpy(str,"qua!");
 	/* now randomize the end of str.. */
-	gettimeofday(&tv,NULL);
-	r=(getpid()<<16)+tv.tv_sec;
+	r = trand32();
 	for (i=0; i<4; i++)
 		str[i+4]=(r<<(i*8))&0377;
 	copykey(&k,&kt);
