@@ -7,14 +7,7 @@
  * modification of this software and in all copies of the supporting
  * documentation for such software.
  *
- * This software is subject to United States export controls.  You may
- * not export it, in whole or in part, or cause or allow such export,
- * through act or omission, without prior authorization from the United
- * States government and written permission from AT&T.  In particular,
- * you may not make any part of this software available for general or
- * unrestricted distribution to others, nor may you disclose this software
- * to persons other than citizens and permanent residents of the United
- * States and Canada. 
+ * This software is subject to United States export controls.
  *
  * THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
  * WARRANTY.  IN PARTICULAR, NEITHER THE AUTHORS NOR AT&T MAKE ANY
@@ -32,9 +25,6 @@
 #include "admproto.h"
 #include "cfs.h"
 #include "shs.h"
-#ifdef SOLARIS2X
-#define getwd(b)        getcwd(b, 1024)
-#endif
 
 main(argc,argv)
      int argc;
@@ -74,7 +64,7 @@ main(argc,argv)
 		exit(1);
 	}
 	if (*argv[0]!='/') {
-		if (getwd(buf) == NULL) {
+		if (getcwd(buf,1024) == NULL) {
 			fprintf(stderr,"Can't stat current directory\n");
 			exit(1);
 		}
@@ -129,6 +119,7 @@ main(argc,argv)
 		exit(1);
 	}
 
+	newkey.cipher=ciph;
 	if ((pw=getpassword("New passphrase:"))==NULL) {
 		fprintf(stderr,"Can't get key\n");
 		exit(1);
@@ -198,12 +189,10 @@ checkkey(path,ak)
 		fclose (fp);
 		return 0;
 	}
-printf("check\n");
 	fclose (fp);
 	cipher(&k,buf,1);        /* note order here */
 	mask_cipher(&k,buf,0);
 	cipher(&k,buf,1);        /* note order here */
-printf("check again\n");
 	if (bcmp(buf,"qua!",4)!=0)
 		return 0;
 	return 1;
