@@ -1,8 +1,8 @@
 /*
  * ESM - Encrypted Session Manager
- * v1.0.1
+ * v1.0.2
  * matt blaze
- * October 1995
+ * December 1995
  */
 
 /* SunOS 5 port by Greg Onufer, based in part on the pty_termios
@@ -187,8 +187,8 @@ main(argc, argv)
 		doshell(mode);
 	else {
 		/* main loop */
-		printf("ESM v1.0.1 - encrypted session manager\n");
-		printf("    by Matt Blaze, AT&T Bell Labs, October 1995\n");
+		printf("ESM v1.0.2 - encrypted session manager\n");
+		printf("    by Matt Blaze, AT&T Bell Labs, December 1995\n");
 		randinit();
 		cipherinit();
 		switch (mode) {
