@@ -124,6 +124,14 @@ old_pwcrunch(b,k)
 		bcopy(k2,k->cfs_admkey_u.des3key.primary2,8);
 		bcopy(k2,k->cfs_admkey_u.des3key.secondary2,8);
 		break;
+	    case CFS_TRUE_THREE_DES: /* this is really 2des */
+		bcopy(k1,k->cfs_admkey_u.tdes3key.primary1,8);
+		bcopy(k1,k->cfs_admkey_u.tdes3key.secondary1,8);
+		bcopy(k2,k->cfs_admkey_u.tdes3key.primary2,8);
+		bcopy(k2,k->cfs_admkey_u.tdes3key.secondary2,8);
+		bcopy(k1,k->cfs_admkey_u.tdes3key.primary3,8);
+		bcopy(k1,k->cfs_admkey_u.tdes3key.secondary3,8);
+		break;
 	    case CFS_MACGUFFIN:
 		bcopy(k1,k->cfs_admkey_u.mcgkey.primary,8);
 		bcopy(k2,&(k->cfs_admkey_u.mcgkey.primary[8]),8);
@@ -155,13 +163,21 @@ new_pwcrunch(b,k)
 	int l;
 	u_char *k1;
 	u_char *k2;
+	u_char *k3;
 	u_char *hash;
+	u_char h1[20];
+	u_char h2[20];
 	
 	if ((l=strlen(b))<3)
 		return -1;
-	hash = shs(b,l);
-	k1 = hash;
-	k2 = &(hash[8]);
+	hash = qshs(b,l);
+	bcopy(hash,h1,20);
+	k1 = h1;
+	k2 = &(h1[8]);
+	/* for true threedes, we do one more hash to get the third key */
+	hash = qshs(h1,20);
+	bcopy(hash,h2,20);
+	k3 = h2;
 	switch (k->cipher) {
 	    case CFS_STD_DES:
 		bcopy(k1,k->cfs_admkey_u.deskey.primary,8);
@@ -173,6 +189,14 @@ new_pwcrunch(b,k)
 		bcopy(k2,k->cfs_admkey_u.des3key.primary2,8);
 		bcopy(k2,k->cfs_admkey_u.des3key.secondary2,8);
 		break;
+	    case CFS_TRUE_THREE_DES:
+		bcopy(k1,k->cfs_admkey_u.tdes3key.primary1,8);
+		bcopy(k1,k->cfs_admkey_u.tdes3key.secondary1,8);
+		bcopy(k2,k->cfs_admkey_u.tdes3key.primary2,8);
+		bcopy(k2,k->cfs_admkey_u.tdes3key.secondary2,8);
+		bcopy(k3,k->cfs_admkey_u.tdes3key.primary3,8);
+		bcopy(k3,k->cfs_admkey_u.tdes3key.secondary3,8);
+		break;
 	    case CFS_MACGUFFIN:
 		bcopy(k1,k->cfs_admkey_u.mcgkey.primary,16);
 		bcopy(k1,k->cfs_admkey_u.mcgkey.secondary,16);
@@ -226,6 +250,23 @@ decrypt_key(k,ek)
 		bcopy(&(ek[8]),k->cfs_admkey_u.des3key.primary2,8);
 		bcopy(&(ek[8]),k->cfs_admkey_u.des3key.secondary2,8);
 		break;
+	    case CFS_TRUE_THREE_DES:
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[0]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[0]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[0]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[8]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[8]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[8]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[16]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[16]),1);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[16]),1);
+		bcopy(&(ek[0]),k->cfs_admkey_u.tdes3key.primary1,8);
+		bcopy(&(ek[0]),k->cfs_admkey_u.tdes3key.secondary1,8);
+		bcopy(&(ek[8]),k->cfs_admkey_u.tdes3key.primary2,8);
+		bcopy(&(ek[8]),k->cfs_admkey_u.tdes3key.secondary2,8);
+		bcopy(&(ek[16]),k->cfs_admkey_u.tdes3key.primary3,8);
+		bcopy(&(ek[16]),k->cfs_admkey_u.tdes3key.secondary3,8);
+		break;
 	    case CFS_MACGUFFIN:
 		mcg_keyset(k->cfs_admkey_u.mcgkey.primary,&mk);
 		mcg_block_decrypt(&mk,&(ek[0]));
@@ -285,6 +326,17 @@ encrypt_key(k,ek)
 		q_block_cipher(k->cfs_admkey_u.des3key.primary2,&(ek[8]),1);
 		q_block_cipher(k->cfs_admkey_u.des3key.primary1,&(ek[8]),0);
 		break;
+	    case CFS_TRUE_THREE_DES:
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[0]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[0]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[0]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[8]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[8]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[8]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary1,&(ek[16]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary2,&(ek[16]),0);
+		q_block_cipher(k->cfs_admkey_u.tdes3key.primary3,&(ek[16]),0);
+		break;
 #ifdef NOTDEF		
 	    case CFS_BLOWFISH:
 		Blowfish_Initialize(k->cfs_admkey_u.blowkey.primary,16,&bk);
